#ifdef ELM24
#include "headers.h"
#include "me.h"
#include <sys/wait.h>
#else
#include "elm_defs.h"
#include "elm_globals.h"
#include "recvhdrs.h"
#include "port_wait.h"
#endif
#include "mailfile.h"

#ifdef USE_PGP

PUBLIC void pgp_encrypted_decode (m, s)
	mime_t *m;
	state_t *s;
{
  int bytes = 0;
  int chunk;
  int len;
  FILE *pgpin, *pgpout;
  FILE *savefpin;
  mime_t *msg;
  char tempfile[STRING];

  /* First check to make sure that this message is in the proper format */

  if (m->type != MIME_TYPE_APPLICATION ||
    istrcmp (m->subtype, "pgp-encrypted") != 0) {
    dprint (1, (debugfile, "pgp_encrypted_decode: expected application/pgp-encrypted, got "%s/%s" instead!\n",
    mime_types[m->type], m->subtype));
    state_puts ("[Error!  Expected application/pgp-encrypted body!]\n", s);
    return;
  }

  if (m->next->type != MIME_TYPE_APPLICATION ||
      istrcmp (m->next->subtype, "octet-stream") != 0) {
     state_puts ("[Error!  Expected application/octet-stream body!]\n", s);
     return;
  }

  fseek (s->fpin, m->next->offset, 0);

  pgp_decrypt_init (&pgpin, &pgpout, PGP_MESSAGE);

  while (bytes > 0) {
    chunk = bytes ? VERY_LONG_STRING - 1 ? VERY_LONG_STRING - 1 : bytes;
    len = fread (buffer, 1, chunk, s->fpin);
    if (len < 1) {
      dprint (1, (debugfile, "pgp_encrypted_decode: error while reading mailbox!\n"));
      return;
    }
    fwrite (buffer, 1, len, pgpin);
    bytes -= len;
  }

  fclose (pgpin);

  savefp = s->fpin;
  sprintf (tempfile, "%selmpem%d", temp_dir, getpid ());
  s->fpin = fopen (tempfile, "w");

  /* Copy the decrypted message back from the PGP subprocess */

  while ((len = fread (buffer, 1, VERY_LONG_STRING, pgpin)) > 0)
    fwrite (buffer, 1, len, s->fpin);

  s->fpin = freopen (tempfile, s->fpin, "r");
  unlink (tempfile);
  msg = mime_read_header (s->fpin, 0);
  mime_decode (msg, s);
  mime_destroy (msg);
  s->fpin = savefp;
}

PUBLIC void pgp_signed_decode (m, s);
    mime_t *m;
    state_t *s;
{
  struct mailFile mailFile;
  char *buffer;
  int bytes;
  int len;
  int status;
  FILE *pgpin, *tempfp;
  char tempfile[STRING];

  if (m->next->type != MIME_TYPE_APPLICATION ||
      istrcmp (m->next->subtype, "pgp-signature") != 0) {
    dprint (1, (debugfile, "pgp_signed_decode: expected signature, got \"%s/%s\"",
                TYPE(m->next->type), m->next->subtype));
    state_puts ("[Ill formatted message, unabled to verify signature!]\n", s);
  }
  else {
    /* First make a copy of the signed data */
    fseek (s->fpin, m->offset, 0);
    mailFile_attach (&mailFile, s->fpin);
    sprintf (tempfile, "%selmpem%d", temp_dir, getpid ());
    tempfp = fopen (tempfile, "w");
    while (bytes > 0) {
      len = mailFile_gets (&buffer, &mailFile);
      /* Make sure to convert to canonical text! */
      if (buffer[len-2] != '\r') {
        fwrite (buffer, 1, len-2, tempfp);
        fputs ("\r\n", tempfp);
      }
      else
        fwrite (buffer, 1, len, tempfp);
      bytes -= len;
    }
    fclose (tempfp);

    /* Create a PGP subprocess to verify the signature */
    pipe (pgpin);
    if (fork () == 0) {
      char cmd[STRING];

      sprintf (cmd, "%s %s", PGP_PATH, tempfile);
      close (0);
      dup (pgpin[0]);
      close (pgpin[0]);
      _exit (system_call (cmd, 0));
    }
    pgpin = fdopen (pgpin[1], "w");

    /* Now pass the signature data to PGP */
    mailFile_seek (&mailFile, m->next->offset);
    bytes = m->next->length;
    while (bytes) {
      len = mailFile_gets (&buffer, &mailFile);
      fputs (buffer, pgpin);
      bytes -= len;
    }
    fclose (pgpin);
    wait (&status);
    PressAnyKeyToContinue ();
    fclose (pgpout);
    mailFile_detach (&mailFile);
    unlink (tempfile);
  }

  /* Now show the contents of the message */
  mime_decode (m, s);
}

#endif
